@@ -304,6 +304,9 @@ class BIDSVariableCollection(object):
             results.extend(vars_)
         return results
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__}{sorted(list(self.variables.keys()))}>"
+
 
 class BIDSRunVariableCollection(BIDSVariableCollection):
     """A container for one or more RunVariables--i.e., Variables that have a
